<?php
/**
 * @copyright   Copyright (c) http://www.manadev.com
 * @license     http://www.manadev.com/license  Proprietary License
 */

namespace Manadev\LayeredNavigation;

use Magento\Framework\UrlInterface;
use Manadev\Core\Exceptions\NotImplemented;

class UrlGenerator
{
    /**
     * @var Engine
     */
    protected $engine;
    /**
     * @var UrlInterface
     */
    protected $urlBuilder;
    /**
     * @var UrlSettings
     */
    protected $urlSettings;
    
    public function __construct(Engine $engine, UrlInterface $urlBuilder, UrlSettings $urlSettings) {
        $this->engine = $engine;
        $this->urlBuilder = $urlBuilder;
        $this->urlSettings = $urlSettings;
    }

    /**
     * @return string
     */
    public function getClearAllUrl() {
        $queryParameters = [];
        foreach ($this->engine->getAppliedFilters() as $engineFilter) {
            $queryParameters[$engineFilter->getFilter()->getData('param_name')] = null;
        }

        return $this->removeAdditionalParamsFromUrl($this->getUrl($queryParameters));
    }

    /**
     * @param EngineFilter $engineFilter
     * @param $item
     * @return string
     */
    public function getAddItemUrl(EngineFilter $engineFilter, $item) {
        $combinedValues = $engineFilter->getAppliedOptions() ?: [];

        if (!is_array($combinedValues)) {
            $combinedValues = [$combinedValues];
        }
        
        if (!in_array($item['value'], $combinedValues)) {
            $combinedValues[] = $item['value'];
        }

        $combinedValues = implode($this->urlSettings->getMultipleValueSeparator(), $combinedValues);

        return $this->removeAdditionalParamsFromUrl($this->getUrl([$engineFilter->getFilter()->getData('param_name') => $combinedValues]));
    }

    /**
     * @param EngineFilter $engineFilter
     * @param $item
     * @return string
     */
    public function getRemoveItemUrl(EngineFilter $engineFilter, $item) {
        $combinedValues = $engineFilter->getAppliedOptions() ?: [];
        if (!is_array($combinedValues)) {
            $combinedValues = [$combinedValues];
        }

        if (($index = array_search($item['value'], $combinedValues)) !== false) {
            unset($combinedValues[$index]);
        }

        if (!count($combinedValues)) {
            $combinedValues = null;
        }
        else {
            $combinedValues = implode($this->urlSettings->getMultipleValueSeparator(), $combinedValues);
        }

        return $this->removeAdditionalParamsFromUrl($this->getUrl([$engineFilter->getFilter()->getData('param_name') => $combinedValues]));
       
    }

    protected function getUrl($queryParameters) {
        return $this->urlBuilder->getUrl('*/*/*', [
            '_current' => true,
            '_use_rewrite' => true,
            '_query' => $queryParameters,
            '_escape' => true,
        ]);
    }

    public function getReplaceItemUrl(EngineFilter $engineFilter, $item) {
        return $this->removeAdditionalParamsFromUrl($this->getUrl([$engineFilter->getFilter()->getData('param_name') => $item['value']]));
    }
    
    public function removeAdditionalParamsFromUrl($url) {
        
        $returnUrl = $url;
        
        $url = htmlspecialchars_decode($url);

        $parsed = parse_url($url);
        
        if (isset($parsed['query'])) {
         
            $query = $parsed['query'];

            parse_str($query, $params);

            unset($params['p']);
            unset($params['is_ajax']);
            $string = http_build_query($params);

            if ($string) {
                $returnUrl = $parsed["scheme"] . "://" . $parsed["host"] . $parsed["path"] . "?" . $string;
            }
            else {
                $returnUrl = $parsed["scheme"] . "://" . $parsed["host"] . $parsed["path"];
            }   
        }

        return $returnUrl;
    }
}